local random = math.random

local MAPS  = nil
local NODES = nil
local BUBBLE_TYPE = 6
-- 网格行数
local gridRows = 6
-- 网格列数
local gridColumns = 4
-- 飞行泡泡个数
local bubbleCount = 8
-- 消失后多长时间回到场景
local renewTime = 0.8

-- 动画名称、帧数、动画时间
local cartoonNames  = {"blue_pp","green_pp","pink_pp","purple_pp","red_pp","yellow_pp","pop_pp"}
local cartoonFrames = {17,17,17,17,17,17,11}
local cartoonTimes  = {1/24,1/24,1/24,1/24,1/24,1/24,1/11}
-- 飞行动作持续时间
local flightTimes = {1.5, 2.0, 2.5, 3.0, 3.5}

local xInitPos = -300
local yInitPos = -300

local BubbleScene = class("BubbleScene", function()
    return display.newScene("BubbleScene")
end)

function BubbleScene:ctor()      
    self:addBG(BG_BUBBLE)
end

function BubbleScene:onEnter()
    if device.platform == "android" then
        self:performWithDelay(function()
            local layer = display.newLayer()
            layer:addKeypadEventListener(function(event)
                if event == "back" then app.exit() end
            end)
            layer:setKeypadEnabled(true)
            self:addChild(layer)            
        end, 0.5)
    end        
end

function BubbleScene:onExit()
    
end
-- 场景清理资源时自动调用此方法
function BubbleScene:onCleanup()
    self:destruct()
end
-- 添加场景背景
function BubbleScene:addBG(bgImage)
    -- display.removeSpriteFrameByImageName(bgImage)
    display.addImageAsync(bgImage, function(event, texture)
        local bg = display.newSprite(bgImage, display.cx, display.cy)
        -- TODO
        bg:setOpacity(0)
        self:addChild(bg)
        self:markAutoCleanupImage(bgImage)
        self.background = bg
        self.bginit = true

        local action = CCFadeIn:create(FADEIN_TIME)
        transition.execute(bg, action, {onComplete = function() self:onInit() end})
    end)    
end

function BubbleScene:onInit()
    -- 缓存游戏资源文件
    display.addSpriteFramesWithFile(PP_SRC[1][1], PP_SRC[1][2])
    self:markAutoCleanupImage(PP_SRC[1][2])
    display.addSpriteFramesWithFile(PP_SRC[2][1], PP_SRC[2][2])    
    self:markAutoCleanupImage(PP_SRC[2][2])
    display.addSpriteFramesWithFile(PP_SRC[3][1], PP_SRC[3][2])
    self:markAutoCleanupImage(PP_SRC[3][2])
    display.addSpriteFramesWithFile(PP_SRC[4][1], PP_SRC[4][2])
    self:markAutoCleanupImage(PP_SRC[4][2])
    display.addSpriteFramesWithFile(PP_SRC[5][1], PP_SRC[5][2])
    self:markAutoCleanupImage(PP_SRC[5][2])
    display.addSpriteFramesWithFile(PP_SRC[6][1], PP_SRC[6][2])
    self:markAutoCleanupImage(PP_SRC[6][2])
    display.addSpriteFramesWithFile(PP_SRC[7][1], PP_SRC[7][2])
    self:markAutoCleanupImage(PP_SRC[7][2])

    -- 场景游戏内容初始化
    self:createGrid(gridRows, gridColumns)
    self:addNodeAndCache()
    self:addNodeBubbles(MAPS, bubbleCount)
    self:initFlightBubbles(bubbleCount)
    self:addPopBubbles(BUBBLE_TYPE + 1, bubbleCount)
    self.gameinit = true

    -- 音乐播放器初始化
    self.musiclist = self:addMusicList(DB_PATH, SQL_QUERY)
    if self.musiclist == nil then 
        self:addDialogBox(DIALOG_BG, DIALOG_FG, DIALOG_BTN)
    else
        self.firstMusicIdx = 1
        self.lastMusicIdx = #self.musiclist
        self.currmusic = 1
        self.playmode = LIST_MODE
        self:playSong(self.musiclist, self.currmusic)
    end
        
    --self:addMiniMusicPanel(MENU_SMALL)
    --self:addMusicPlayerPanel(MENU_PLAY, MENU_ORDER_LIST, MENU_ALL_LOOP, MENU_ONE_LOOP, MENU_PREV, MENU_START, MENU_PAUSE, MENU_NEXT, MENU_PLAYLIST)
    self.musicinit = true

end
-- 新建并添加渲染节点和缓存
function BubbleScene:addNodeAndCache()
    NODES = {}
    for idx = 1, #cartoonNames, 1 do 
        self:addRenderNode(idx)
        self:setCartoonCache(idx)
    end    
end
-- 渲染节点
function BubbleScene:addRenderNode(nodeID)
    local image  = PP_SRC[nodeID][2]
    local node   = display.newBatchNode(image)
    node.bubbles = {}
    self:addChild(node)
    table.insert(NODES, node)
end
-- 设置动画缓存
function BubbleScene:setCartoonCache(cacheID)
    local pattern     = cartoonNames[cacheID] .. "_%02d.png"    
    local frameLength = cartoonFrames[cacheID]
    local frames      = display.newFrames(pattern, 1, frameLength)
    local time        = cartoonTimes[cacheID]
    local animation   = display.newAnimation(frames, time)
    local cacheName   = cartoonNames[cacheID]
    display.setAnimationCache(cacheName, animation)    
end

function BubbleScene:createGrid(rows, columns)
    local wCell = display.width / columns
    local hCell = display.height / rows
    local wHalfCell = wCell * 0.5
    local hHalfCell = hCell * 0.5

    local maps = {}
    for row = 0, rows - 1 do        
        for column = 0, columns - 1 do            
            local x = toint(column * wCell + wHalfCell)
            local y = toint(row * hCell + hHalfCell)
            local p = CCPoint(x, y)
            maps[#maps + 1] = p
        end
    end

    MAPS = maps
end

function BubbleScene:addPopBubbles(typeid, bubbleCount)
    local node = NODES[typeid]
    for idx = 1, bubbleCount, 1 do
        self:addPopBubble(node, typeid, xInitPos, yInitPos)
    end
end
-- 添加爆裂泡泡
function BubbleScene:addPopBubble(parent, parentID, x, y)
    local cacheName = cartoonNames[parentID]
    local bubbleName = "#" .. cacheName .. "_01.png"
    local bubble = display.newSprite(bubbleName, x, y)
    bubble:setVisible(false)
    table.insert(parent.bubbles, bubble)
    bubble.id = #parent.bubbles
    parent:addChild(bubble)
end
-- 每个渲染层都添加指定的泡泡个数
function BubbleScene:addNodeBubbles(maps, bubbleCount)
    for nodeidx = 1, BUBBLE_TYPE, 1 do
        -- local pts = self:getMultiRandomPos(maps, bubbleCount)
        local node = NODES[nodeidx]
        for idx = 1, bubbleCount, 1 do            
            self:addNodeBubble(node, nodeidx, xInitPos, yInitPos)
        end
    end
end

function BubbleScene:addNodeBubble(parent, parentID, x, y)
    local cacheName = cartoonNames[parentID]
    local bubbleName = "#" .. cacheName .. "_01.png"
    local bubble = display.newSprite(bubbleName, x, y)
    bubble:setVisible(false)
    bubble:setTouchEnabled(true)
    bubble:addTouchEventListener(function(event, x, y)
        if event == "began" then
            local touchIn = bubble:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y))
            if bubble:isVisible() and touchIn then
                bubble:setVisible(false)
                self:initBubbleActionAndAnimation(bubble, parentID)
                local tx, ty = bubble:getPosition()
                local node = NODES[BUBBLE_TYPE + 1]
                local popBubble = node.bubbles[bubble.id]
                popBubble:setPosition(tx,ty)
                popBubble:setVisible(true)
                local popCacheName = cartoonNames[BUBBLE_TYPE + 1]
                local animation = display.getAnimationCache(popCacheName)
                popBubble:playAnimationOnce(animation, false, self:renewBubble(popBubble))
            end
            
            return cc.TOUCH_BEGAN_SWALLOWS
        elseif event == "moved" then
        elseif event == "ended" then            
        else            
        end            
    end)    

    table.insert(parent.bubbles, bubble)
    bubble.id = #parent.bubbles
    parent:addChild(bubble)    
end
-- 将已经消失的飞行泡泡重新添加到场景中
function BubbleScene:renewBubble(popBubble)
    self:performWithDelay(function()
        local tx, ty = popBubble:getPosition()
        local bubbleid = popBubble.id
        popBubble:setVisible(false)
        popBubble:setPosition(xInitPos, yInitPos)
        self:initBubbleActionAndAnimation(popBubble, BUBBLE_TYPE + 1)        

        local nodeid = random(1, BUBBLE_TYPE)
        local node = NODES[nodeid]
        local pt = self:getRandomPos(MAPS, tx, ty)
        local flightBubble = node.bubbles[bubbleid]
        flightBubble:setPosition(pt.x, pt.y)
        flightBubble:setVisible(true)
        local flightCacheName = cartoonNames[nodeid]
        flightBubble:playForever(display.getAnimationCache(flightCacheName))
        self:setRandomMove(flightBubble, MAPS)
    end, renewTime)
end
-- 初始化泡泡的动作和动画
function BubbleScene:initBubbleActionAndAnimation(bubble, typeid)
    -- 停止所有动作和动画
    transition.stopTarget(bubble)
    local cacheName = cartoonNames[typeid]
    -- 还原初始状态并静止
    bubble:setDisplayFrameWithAnimationName(cacheName, 0)        
end
-- 初始化屏幕中的彩色泡泡
function BubbleScene:initFlightBubbles(bubbleCount)
    for idx = 1, bubbleCount, 1 do
        local rId = random(1, BUBBLE_TYPE)
        local node = NODES[rId]
        local bubble = node.bubbles[idx]
        bubble:setVisible(true)
        local cacheName = cartoonNames[rId]
        bubble:playForever(display.getAnimationCache(cacheName))        
        self:setRandomMove(bubble, MAPS)
    end
end

-- 设置随机飞行动作
function BubbleScene:setRandomMove(bubble, maps)
    local index      = random(1, #flightTimes)
    local time       = flightTimes[index]
    local tx, ty     = bubble:getPosition()
    local moveAction = CCMoveTo:create(time, self:getRandomPos(maps, tx, ty))

    transition.execute(bubble, moveAction, {onComplete = function() 
        return self:setRandomMove(bubble, maps) 
    end})
end
-- 从数据库中读取播放列表
function BubbleScene:addMusicList(dburl, sqlquery)
    if not io.exists(dburl) then return nil end
    local sqlite3 = require("lsqlite3")
    local db = sqlite3.open(dburl)
    local playlist = {}
    for folder, file, song in db:urows(sqlquery) do
        table.insert(playlist, {folder, file, song})
    end        
    db:close()
    -- 数据库可能出现没有歌曲的情况
    if #playlist == 0 then playlist = nil end
    return playlist
end
-- 播放歌曲
function BubbleScene:playSong(list, index)
    if list == nil then return end

    local songName = list[index][3]
    if self.title == nil then 
        self:addTitle(TOP_BG, songName)
    else
        self:changeTitle(songName)
    end

    local folderPath = list[index][1]
    -- audio.preloadMusic(folderPath)
    local songFile = folderPath .. list[index][2]
	--wm
    --audio.preloadMusic(songFile)
    --audio.playMusic(songFile, false)
end
-- 播放完成当前歌曲后，该如何操作
function BubbleScene:doneCurrentSong(mode)
    if mode == LIST_MODE then
        self.currmusic = self.currmusic + 1
        if self.currmusic > self.lastMusicIdx then
            self.currmusic = self.lastMusicIdx
            audio.stopMusic()
        else
            self:playSong(self.musiclist, self.currmusic)
        end

    elseif mode == ALLLOOP_MODE then
        self.currmusic = self.currmusic + 1
        if self.currmusic > self.lastMusicIdx then 
            self.currmusic = self.firstMusicIdx 
        end
        self:playSong(self.musiclist, self.currmusic)
    elseif mode == ONELOOP_MODE then
        self:playSong(self.musiclist, self.currmusic)
    end    
end
-- 添加歌曲标题
function BubbleScene:addTitle(titleImage, title)
--[[
    local topbg = display.newSprite(titleImage)
    local size  = topbg:getContentSize()
    topbg:setPosition(display.left, display.top - (size.height * 0.5))
    topbg:setScaleX((display.width / size.width) * 2)    
    self:addChild(topbg, TITLE_BG_LAYER)
    self:markAutoCleanupImage(titleImage)
    self.topbg = topbg
	--wm
    local title = CCLabelTTF:create(title, TITLE_FONT, TITLE_SIZE)        
    local size  = title:getContentSize()
    title:setPosition(display.cx, display.top - (size.height * 0.5))
    title:setColor(TITLE_COLOR)
    self:addChild(title, TITLE_LAYER)
    self.title = title
]]   
end
-- 修改歌曲标题，切换歌曲时用到
function BubbleScene:changeTitle(title)
    self.title:setString(title) 
end
-- 播放器最小化面板
function BubbleScene:addMiniMusicPanel(menuImage)
    local menu = display.newSprite(menuImage)
    local size = menu:getContentSize()
    local tx   = display.cx
    local ty   = size.height * 0.5
    menu:setPosition(tx, ty)
    menu:setTouchEnabled(true)
    menu:addTouchEventListener(function(event, x, y)   
        -- 点击后小菜单不可再次点击，显示播放器动作
        if event == "began" then
            menu:setVisible(false)
            menu:setTouchEnabled(false)
            
            local size = self.playBG:getContentSize()
            local tx   = display.cx
            local ty   = size.height * 0.5            
            local moveAction = CCMoveTo:create(SHOW_MUSICPANEL_TIME, CCPoint(tx,ty))

            --播放器面板展开时的操作
            transition.execute(self.playBG, moveAction, {onComplete = function()                
                -- 定义何时收回播放器面板
                self:setTouchMusicPanel(PANEL_OPEN)
                menu:scheduleUpdate(handler(self, self.onMusicPanelMonitor)) 
            end})            
            return false
        end
    end) 

    self:addChild(menu, MENU_SMALL_LAYER)
    self:markAutoCleanupImage(menuImage)
    self.menu = menu
end
-- 添加播放器
function BubbleScene:addMusicPlayerPanel(playBGImage, playOrderListImage, 
    playAllLoopImage, playOneLoopImage, playPrevImage, playStartImage, playPauseImage, playNextImage, playPlayListImage)

    local playBG = display.newSprite(playBGImage)
    local size   = playBG:getContentSize()
    local tx     = display.cx
    local ty     = -size.height * 0.5
    playBG.runtime = 0
    playBG:setPosition(tx, ty)

    -- 数据库中有播放歌曲时，才需要监听正在播放的歌曲状态
    if self.musiclist ~= nil then
        playBG:scheduleUpdate(handler(self, self.onMusicMonitor)) 
    end

    self:addChild(playBG, MENU_PLAY_LAYER)
    self:markAutoCleanupImage(playBGImage)
    self.playBG  = playBG

    self.playOrderList  = self:addMusicPlay(playBG, playOrderListImage, "orderlist", true, true, 82, 35)    
    self.playAllLoop    = self:addMusicPlay(playBG, playAllLoopImage,   "allloop",   true, false, 82, 35)    
    self.playOneLoop    = self:addMusicPlay(playBG, playOneLoopImage,   "oneloop",   true, false, 82, 35)
    self.playPrev       = self:addMusicPlay(playBG, playPrevImage,      "prev",      true, true, 114, 106)
    self.playPause      = self:addMusicPlay(playBG, playStartImage,     "pause",     true, false, 218, 125)
    self.playStart      = self:addMusicPlay(playBG, playPauseImage,     "start",     true, true, 218, 125)
    self.playNext       = self:addMusicPlay(playBG, playNextImage,      "next",      true, true, 307, 106)
    self.playList       = self:addMusicPlay(playBG, playPlayListImage,  "list",      true, true, 347, 35)
    self.selectedPlayer = nil
end
-- 添加播放器的播放按钮
function BubbleScene:addMusicPlay(parent, image, name, bTouch, bVisible, tx, ty, opacity)
    local sprite = display.newSprite(image, tx, ty)
    sprite.name = name
    sprite.touchstatus = bTouch
    if opacity ~= nil then sprite:setOpacity(opacity) end
    sprite:setVisible(bVisible)
    sprite:setTouchEnabled(bTouch)    
    sprite:addTouchEventListener(handler(self, self.onTouchEvent))
    parent:addChild(sprite)
    self:markAutoCleanupImage(image)
    return sprite
end
-- 设置播放、暂停按钮的可见状态
function BubbleScene:setMusicPlayStatus(status)
    if status == PLAY_STATUS then
        self.playStart:setVisible(true)
        self.playPause:setVisible(false)        
    elseif status == PAUSE_STATUS then
        self.playPause:setVisible(true)        
        self.playStart:setVisible(false)        
    end
end
-- 设置列表、列表循环、单曲循环的可见状态
function BubbleScene:setMusicPlayMode(mode)
    if mode == LIST_MODE then
        self.playmode = LIST_MODE
        self.playOrderList:setVisible(true)
        self.playAllLoop:setVisible(false)
        self.playOneLoop:setVisible(false)
    elseif mode == ALLLOOP_MODE then
        self.playmode = ALLLOOP_MODE
        self.playAllLoop:setVisible(true)
        self.playOrderList:setVisible(false)        
        self.playOneLoop:setVisible(false)        
    elseif mode == ONELOOP_MODE then
        self.playmode = ONELOOP_MODE
        self.playOneLoop:setVisible(true)
        self.playAllLoop:setVisible(false)
        self.playOrderList:setVisible(false)        
    end
end
-- 记忆播放器按键的状态
function BubbleScene:setTouchMusicPanel(mode)
    if mode == PANEL_OPEN then
        self.playOrderList:setTouchEnabled(self.playOrderList.touchstatus)
        self.playAllLoop:setTouchEnabled(self.playAllLoop.touchstatus)
        self.playOneLoop:setTouchEnabled(self.playOneLoop.touchstatus)
        self.playPrev:setTouchEnabled(self.playPrev.touchstatus)
        self.playPause:setTouchEnabled(self.playPause.touchstatus)
        self.playStart:setTouchEnabled(self.playStart.touchstatus)
        self.playList:setTouchEnabled(self.playList.touchstatus)
        self.playNext:setTouchEnabled(self.playNext.touchstatus)

    elseif mode == PANEL_CLOSE then
        self.playOrderList.touchstatus = self.playOrderList:isTouchEnabled()
        self.playOrderList:setTouchEnabled(false)

        self.playAllLoop.touchstatus = self.playAllLoop:isTouchEnabled()
        self.playAllLoop:setTouchEnabled(false)        

        self.playOneLoop.touchstatus = self.playOneLoop:isTouchEnabled()
        self.playOneLoop:setTouchEnabled(false)        

        self.playPrev.touchstatus = self.playPrev:isTouchEnabled()
        self.playPrev:setTouchEnabled(false)                

        self.playPause.touchstatus = self.playPause:isTouchEnabled()
        self.playPause:setTouchEnabled(false)                        

        self.playStart.touchstatus = self.playStart:isTouchEnabled()
        self.playStart:setTouchEnabled(false)                                

        self.playList.touchstatus = self.playList:isTouchEnabled()
        self.playList:setTouchEnabled(false)

        self.playNext.touchstatus = self.playNext:isTouchEnabled()
        self.playNext:setTouchEnabled(false)
    end
end
-- 控制播放器隐藏
function BubbleScene:onMusicPanelMonitor(dt)
    self.playBG.runtime = self.playBG.runtime + dt
    -- HIDE_MUSICPANEL_TIME 秒钟后没有播放器操作就隐藏播放器
    if self.playBG.runtime > HIDE_MUSICPANEL_TIME then
        local size = self.playBG:getContentSize()
        local tx   = display.cx
        local ty   = -size.height * 0.5            
        local moveAction = CCMoveTo:create(1.0, CCPoint(tx,ty))

        self:setTouchMusicPanel(PANEL_CLOSE)
        -- 播放器面板回收之后定义的操作
        transition.execute(self.playBG, moveAction, {onComplete = function()
             self.menu:setTouchEnabled(true)
             self.menu:setVisible(true)
        end})
                
        self.menu:unscheduleUpdate()
        self.playBG.runtime = 0
    end
end
-- 监控歌曲播放完成后，如何操作
function BubbleScene:onMusicMonitor(dt)
    if self.musiclist == nil then return end

    local bPlaying = audio.isMusicPlaying()
    if bPlaying then return end
    -- 根据播放模式，列表、列表循环、单曲循环来确定
    -- 下一首歌该怎么播
    self:doneCurrentSong(self.playmode)
end

-- 播放器按钮触摸操作
function BubbleScene:onTouchEvent(event, x, y)
    -- 数据库没有歌曲，所以按钮失效
    if self.musiclist == nil then return false end
    local target = nil
    if event == "began" then
        target = self:getTouchTarget(x, y)
        if target == nil then 
            return false
        else 
            target:setOpacity(128)
            self.selectedPlayer = target
            return true
        end

    elseif event == "moved" then
        target = self:getTouchTarget(x, y)
        if target ~= self.selectedPlayer then
            self.selectedPlayer:setOpacity(255)
            self.selectedPlayer = nil
            return false
        else 
        end

    elseif event == "ended" and self.selectedPlayer ~= nil then
        self.selectedPlayer:setOpacity(255)
        local name = self.selectedPlayer.name
        self:onClickMusicButton(name)
        -- 按钮被操作，重新计算隐藏播放器的时间
        self.playBG.runtime = 0        
        -- 重置被选按钮
        self.selectedPlayer = nil
    end
end
-- 播放器组件确定点击操作
function BubbleScene:onClickMusicButton(name)
    if name == "start" then
        audio.pauseMusic()
        self:setMusicPlayStatus(PAUSE_STATUS)
    elseif name == "pause" then
        audio.resumeMusic()
        self:setMusicPlayStatus(PLAY_STATUS)
    elseif name == "prev" then
        self.currmusic = self.currmusic - 1
        if self.currmusic < self.firstMusicIdx then 
            self.currmusic = self.lastMusicIdx 
        end

        self:playSong(self.musiclist, self.currmusic)

        if self.playPause:isVisible() then
           self:setMusicPlayStatus(PLAY_STATUS)
        end
    elseif name == "next" then
        self.currmusic = self.currmusic + 1
        if self.currmusic > self.lastMusicIdx then 
            self.currmusic = self.firstMusicIdx 
        end

        self:playSong(self.musiclist, self.currmusic)

        if self.playPause:isVisible() then
           self:setMusicPlayStatus(PLAY_STATUS)
        end
    elseif name == "orderlist" then
        self:setMusicPlayMode(ALLLOOP_MODE)
    elseif name == "allloop" then
        self:setMusicPlayMode(ONELOOP_MODE)
    elseif name == "oneloop" then
        self:setMusicPlayMode(LIST_MODE)
    elseif name == "list" then
        -- TODO 跳转到APK列表页面
    end    
end
-- 获取被点击的播放器按钮
function BubbleScene:getTouchTarget(x,y)
    if  self.playOrderList:isTouchEnabled() and 
        self.playOrderList:isVisible() and
        self.playOrderList:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then 
        return self.playOrderList 
    elseif self.playAllLoop:isTouchEnabled() and 
        self.playAllLoop:isVisible() and
        self.playAllLoop:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playAllLoop
    elseif self.playOneLoop:isTouchEnabled() and 
        self.playOneLoop:isVisible() and
        self.playOneLoop:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playOneLoop
    elseif self.playPrev:isTouchEnabled() and 
        self.playPrev:isVisible() and
        self.playPrev:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playPrev
    elseif self.playPause:isTouchEnabled() and 
        self.playPause:isVisible() and
        self.playPause:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playPause
    elseif self.playStart:isTouchEnabled() and 
        self.playStart:isVisible() and
        self.playStart:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playStart
    elseif self.playList:isTouchEnabled() and 
        self.playList:isVisible() and
        self.playList:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playList
    elseif self.playNext:isTouchEnabled() and 
        self.playNext:isVisible() and
        self.playNext:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playNext                        
    else
        return nil
    end
end
-- 清理程序
function BubbleScene:destruct()
    if self.musicinit then
        self:removeMusicPlayer()
    end

    if self.gameinit then
        self:removeGameContent()
    end

    if self.bginit then
        transition.stopTarget(self.background)
        self.background:removeSelf(true)
    end

    audio.stopAllSounds()
    if audio.isMusicPlaying() then audio.stopMusic(true) end

    transition.stopTarget(self)
    self:removeAllChildrenWithCleanup(true)
end

function BubbleScene:removeGameContent()
    local node    = nil
    local bubbles = nil
    local bubble  = nil

    for nodeid = 1, #NODES, 1 do
        node    = NODES[nodeid]
        bubbles = node.bubbles

        for bubbleid = 1, #bubbles, 1 do
            bubble = bubbles[bubbleid]
            transition.stopTarget(bubble)

            if nodeid <= BUBBLE_TYPE then
                self:removeTouchTarget(bubble, true)
            else
                bubble:removeSelf(true)
            end
            bubbles[bubbleid] = 0
        end

        node.bubbles = nil
        node:removeSelf(true)
        NODES[nodeid] = 0
    end

    NODES = nil
end
-- 场景中删除播放器
function BubbleScene:removeMusicPlayer()
    transition.stopTarget(self.menu)
    self.menu:unscheduleUpdate()
    self:removeTouchTarget(self.menu, true)
    self.menu = nil

    self:removeTouchTarget(self.playOrderList, true)
    self.playOrderList = nil

    self:removeTouchTarget(self.playAllLoop, true)
    self.playAllLoop = nil

    self:removeTouchTarget(self.playOneLoop, true)
    self.playOneLoop = nil    

    self:removeTouchTarget(self.playPrev, true)
    self.playPrev = nil     

    self:removeTouchTarget(self.playPause, true)
    self.playPause = nil

    self:removeTouchTarget(self.playStart, true)
    self.playStart = nil

    self:removeTouchTarget(self.playNext, true)
    self.playNext = nil     

    self:removeTouchTarget(self.playList, true)
    self.playList = nil     
    self.selectedPlayer = nil

    transition.stopTarget(self.playBG)
    self.playBG:unscheduleUpdate()
    self:removeTouchTarget(self.playBG, true)
    self.playBG = nil    


    -- 虽然self.title继承CCSprite，
    -- 但是使用self.title:removeSelf(true)会报错，原因何在？
    self.title:removeFromParentAndCleanup(true)
    self.title = nil
              
    -- self.topbg:removeSelf(true)
    -- self.topbg = nil           
    self.musiclist = nil
end
-- 删除播放按钮
function BubbleScene:removeTouchTarget(sprite, bclean)
    sprite:setTouchEnabled(false)
    sprite:removeTouchEventListener()
    sprite:removeSelf(bclean)
end

-- 获得当前x,y在map中的索引号
function BubbleScene:getMapsIndex(x, y)
    local wCell = display.width / gridColumns
    local hCell = display.height / gridRows

    local row    = toint(y / hCell)
    local column = toint(x / wCell)

    local index = (row - 1) * gridColumns + column
    return index    
end

-- 获得随机map中的位置
function BubbleScene:getRandomPos(maps, x, y)
    local index     = self:getMapsIndex(x, y)
    local cloneMaps = clone(maps)
    table.remove(cloneMaps, index)
    index = random(1, #cloneMaps)
    return cloneMaps[index]
end

-- 获得多个随机map中的位置
function BubbleScene:getMultiRandomPos(maps, posCount)
    local count = (posCount >= #maps) and #maps or posCount

    local cloneMaps = clone(maps)
    local multiPos  = {}
    for index = 1, count, 1 do
        local idx = random(1, #cloneMaps)
        local pt  = cloneMaps[idx]

        table.insert(multiPos, pt)
        table.remove(cloneMaps, idx)        
    end

    return multiPos
end
--没有可播放歌曲提示窗口
--[[
function BubbleScene:addDialogBox(bgImage, fgImage, btnImage)
    local node = display.newNode()
    self:addChild(node, DIALOG_LAYER)
    self.dialog = node

    local bg = display.newSprite(bgImage, display.cx, display.cy)
    node:addChild(bg)
    node.bg = bg

    local fg = display.newSprite(fgImage, display.cx, display.cy)
    node:addChild(fg)
    node.fg = fg

    local btn = display.newSprite(btnImage, display.cx + 100, display.cy)
    node:addChild(btn)
    node.btn = btn

    local msg = CCLabelTTF:create(DIALOG_MSG, TITLE_FONT, TITLE_SIZE)
    msg:setPosition(display.cx, display.cy - 100)
    msg:setColor(DIALOG_COLOR)
    node:addChild(msg)
    node.msg = msg

    node:setTouchEnabled(true)
    node:addTouchEventListener(function(event, x, y) 
        if event == "began" then
            local touchIn = btn:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y))
            if not touchIn then return cc.TOUCH_BEGAN_SWALLOWS end

            self:removeDialogBox(bgImage, fgImage, btnImage)
            return cc.TOUCH_BEGAN_SWALLOWS
        elseif event == "moved" then
        elseif event == "ended" then            
        else            
        end 
    end)
end
]]--
function BubbleScene:removeDialogBox(bgImage, fgImage, btnImage)
    self.dialog.msg:removeFromParentAndCleanup(true)
    self.dialog.msg = nil
    self.dialog.btn:removeSelf(true)
    self.dialog.btn = nil
    self.dialog.fg:removeSelf(true)
    self.dialog.fg = nil
    self.dialog.bg:removeSelf(true)
    self.dialog.bg = nil
    self.dialog:setTouchEnabled(false)
    self.dialog:removeTouchEventListener()    
    self.dialog:removeSelf(true)
    self.dialog = nil

    self:markAutoCleanupImage(bgImage)
    self:markAutoCleanupImage(fgImage)
    self:markAutoCleanupImage(btnImage)
end

return BubbleScene