local random = math.random

local NODES = nil
-- 汽车状态
local STOP_STATUS = "stop"
local MOVE_STATUS = "move"
-- 车道x，y值
local drivewayY = {113,334,558,773}
local drivewayX = {-240,940} 
-- 汽车出现、运行时间
local driveTimes  = {2.5, 3.0, 3.5, 4.0, 4.5, 5.0, 5.5}
local dirveDelays = {0.3, 0.5, 0.8, 1.2, 1.5}
-- 动画名称、帧数、动画时间
local cartoonNames  = {"green_car", "pink_car", "red_car", "yellow_car"}
local cartoonFrames = {6,6,6,6}
local cartoonTimes  = {1/60,1/60,1/60,1/60}

local CarScene = class("CarScene", function()
    return display.newScene("CarScene")
end)

function CarScene:ctor()        
    self:addBG(BG_CAR)
end

function CarScene:onEnter()
    if device.platform == "android" then
        self:performWithDelay(function()
            local layer = display.newLayer()            
            layer:addKeypadEventListener(function(event)
                if event == "back" then app.exit() end
            end)
            layer:setKeypadEnabled(true)
            self:addChild(layer)            
        end, 0.5)
    end
end

function CarScene:onInit()
    display.addSpriteFramesWithFile(CAR_SRC[1][1], CAR_SRC[1][2])
    self:markAutoCleanupImage(CAR_SRC[1][2])
    display.addSpriteFramesWithFile(CAR_SRC[2][1], CAR_SRC[2][2])
    self:markAutoCleanupImage(CAR_SRC[2][2])
    display.addSpriteFramesWithFile(CAR_SRC[3][1], CAR_SRC[3][2])
    self:markAutoCleanupImage(CAR_SRC[3][2])
    display.addSpriteFramesWithFile(CAR_SRC[4][1], CAR_SRC[4][2])
    self:markAutoCleanupImage(CAR_SRC[4][2])
      
    self:addNodeAndCache()
    self:addCars()
    self.gameinit = true

    -- 音乐播放器初始化
    self.musiclist = self:addMusicList(DB_PATH, SQL_QUERY)
    if self.musiclist == nil then 
        self:addDialogBox(DIALOG_BG, DIALOG_FG, DIALOG_BTN)
    else
        self.firstMusicIdx = 1
        self.lastMusicIdx = #self.musiclist
        self.currmusic = 1
        self.playmode = LIST_MODE
        self:playSong(self.musiclist, self.currmusic)
    end
        
    --self:addMiniMusicPanel(MENU_SMALL)
    --self:addMusicPlayerPanel(MENU_PLAY, MENU_ORDER_LIST, MENU_ALL_LOOP, MENU_ONE_LOOP, MENU_PREV, MENU_START, MENU_PAUSE, MENU_NEXT, MENU_PLAYLIST)
    self.musicinit = true   
end

function CarScene:onExit()
   
end

-- 场景清理资源时自动调用此方法
function CarScene:onCleanup()
    self:destruct()
end

-- 新建并添加背景
function CarScene:addBG(bgImage)
    -- display.removeSpriteFrameByImageName(bgImage)
    display.addImageAsync(bgImage, function(event, texture)
        local bg = display.newSprite(bgImage, display.cx, display.cy)        
        bg:setOpacity(0)
        self:addChild(bg)
        self:markAutoCleanupImage(bgImage)
        self.background = bg
        self.bginit = true

        local action = CCFadeIn:create(FADEIN_TIME)
        transition.execute(bg, action, {onComplete = function() self:onInit() end})
    end)
end
-- 新建并添加渲染节点和缓存
function CarScene:addNodeAndCache()
    NODES = {}
    for idx = 1, #cartoonNames, 1 do 
        self:addRenderNode(idx)
        self:setCartoonCache(idx)
    end
end
-- 渲染节点
function CarScene:addRenderNode(nodeID)
    local image = CAR_SRC[nodeID][2]
    local node  = display.newBatchNode(image)
    node.movecars = {}
    node.stopcars = {}
    self:addChild(node)
    table.insert(NODES, node)
end
-- 设置动画缓存
function CarScene:setCartoonCache(cacheID)
    local pattern     = cartoonNames[cacheID] .. "_%02d.png"    
    local frameLength = cartoonFrames[cacheID]
    local frames      = display.newFrames(pattern, 1, frameLength)
    local time        = cartoonTimes[cacheID]
    local animation   = display.newAnimation(frames, time)
    local cacheName   = cartoonNames[cacheID]
    display.setAnimationCache(cacheName, animation)    
end
-- 新建并添加汽车
function CarScene:addCars()
    local yclone  = clone(drivewayY)
    for idx = 1, #yclone, 1 do
        local yid = random(1, #yclone)
        local ty = yclone[yid]
        table.remove(yclone, yid)

        for i = 1, #cartoonNames, 1 do
            local xid = random(1, #drivewayX)
            local tx = drivewayX[xid]
            local movecar = self:newMoveCar(i, tx, ty)
            local stopcar = self:newStopCar(i, tx, ty)            
        end

        self:carMoving(idx)
    end
end
-- 新建并添加可播放动画的汽车
function CarScene:newMoveCar(nodeID, x, y)
    local cacheName = cartoonNames[nodeID]
    local carName   = "#" .. cacheName .. "_01.png"
    local movecar   = display.newSprite(carName, x, y)
    movecar.nodeid  = nodeID
    movecar.state   = MOVE_STATUS
    movecar:playForever(display.getAnimationCache(cacheName))
    movecar:setTouchEnabled(true)
    movecar:addTouchEventListener(function(event, x, y)
        if event == "began" then
            local bTouchIn = movecar:getCascadeBoundingBox():containsPoint(CCPoint(x, y))
            if bTouchIn and movecar.state == MOVE_STATUS then
                local tx, ty = movecar:getPosition()
                movecar:setVisible(false)
                movecar.state = STOP_STATUS
                transition.pauseTarget(movecar)

                local parent = NODES[movecar.nodeid]
                local stopcar = parent.stopcars[movecar.id]
                stopcar:setPosition(tx, ty)
                stopcar:flipX(movecar:isFlipX())
                stopcar:setVisible(true)
            end                
            return cc.TOUCH_BEGAN_SWALLOWS            
        end
    end)

    local parent = NODES[nodeID]
    table.insert(parent.movecars, movecar)
    movecar.id = #parent.movecars
    parent:addChild(movecar)
end
-- 新建并添加静止的汽车
function CarScene:newStopCar(nodeID, x, y)
    local carName  = "#" .. cartoonNames[nodeID] .. "_07.png"
    local stopcar  = display.newSprite(carName, x, y)
    stopcar.nodeid = nodeID
    stopcar:setVisible(false)
    stopcar:setTouchEnabled(true)
    stopcar:addTouchEventListener(function(event, x, y)
        if event == "began" then            
            local bTouchIn = stopcar:getCascadeBoundingBox():containsPoint(CCPoint(x, y))            
            local parent  = NODES[stopcar.nodeid]
            local movecar = parent.movecars[stopcar.id]                                            
            if bTouchIn and movecar.state == STOP_STATUS then                 
                local xpos = drivewayX[1]
                stopcar:setPositionX(xpos)
                stopcar:setVisible(false)

                transition.resumeTarget(movecar)
                movecar:setVisible(true)
                movecar.state = MOVE_STATUS
            end
            return cc.TOUCH_BEGAN_SWALLOWS            
        end
    end)  

    local parent = NODES[nodeID]
    table.insert(parent.stopcars, stopcar)
    stopcar.id = #parent.stopcars
    parent:addChild(stopcar)    
end
-- 汽车移动动作
function CarScene:carMoving(index)    
    local nodeidx   = random(1, #NODES)
    local node      = NODES[nodeidx]
    local movecar   = node.movecars[index]
    local delayTime = dirveDelays[random(1, #dirveDelays)]
    self:performWithDelay(function()     
        local tx, ty     = movecar:getPosition()                
        local moveToX    = (tx == drivewayX[2]) and drivewayX[1] or drivewayX[2]
        local actionTime = driveTimes[random(1, #driveTimes)]        
        local action     = CCMoveTo:create(actionTime, CCPoint(moveToX, ty))

        if tx == drivewayX[1] then
            movecar:flipX(false)
        elseif tx == drivewayX[2] then
            movecar:flipX(true)
        end
        
        transition.resumeTarget(movecar)
        movecar:setVisible(true)

        transition.execute(movecar, action, {onComplete = function()
            movecar:setVisible(false)
            transition.pauseTarget(movecar)
            return self:carMoving(movecar.id) 
        end})    
    end, delayTime)
end

-- 从数据库中读取播放列表
function CarScene:addMusicList(dburl, sqlquery)
    if not io.exists(dburl) then return nil end
    local sqlite3 = require("lsqlite3")
    local db = sqlite3.open(dburl)
    local playlist = {}
    for folder, file, song in db:urows(sqlquery) do
        table.insert(playlist, {folder, file, song})
    end        
    db:close()
    -- 数据库可能出现没有歌曲的情况
    if #playlist == 0 then playlist = nil end
    return playlist
end
-- 播放歌曲
function CarScene:playSong(list, index)
    if list == nil then return end

    local songName = list[index][3]
    if self.title == nil then 
        self:addTitle(TOP_BG, songName)
    else
        self:changeTitle(songName)
    end

    local folderPath = list[index][1]
    -- audio.preloadMusic(folderPath)
    local songFile = folderPath .. list[index][2]
	--wm
    --audio.preloadMusic(songFile)
    --audio.playMusic(songFile, false)
end
-- 播放完成当前歌曲后，该如何操作
function CarScene:doneCurrentSong(mode)
    if mode == LIST_MODE then
        self.currmusic = self.currmusic + 1
        if self.currmusic > self.lastMusicIdx then
            self.currmusic = self.lastMusicIdx
            audio.stopMusic()
        else
            self:playSong(self.musiclist, self.currmusic)
        end

    elseif mode == ALLLOOP_MODE then
        self.currmusic = self.currmusic + 1
        if self.currmusic > self.lastMusicIdx then 
            self.currmusic = self.firstMusicIdx 
        end
        self:playSong(self.musiclist, self.currmusic)
    elseif mode == ONELOOP_MODE then
        self:playSong(self.musiclist, self.currmusic)
    end    
end
-- 添加歌曲标题
function CarScene:addTitle(titleImage, title)
--[[
    local topbg = display.newSprite(titleImage)
    local size  = topbg:getContentSize()
    topbg:setPosition(display.left, display.top - (size.height * 0.5))
    topbg:setScaleX((display.width / size.width) * 2)    
    self:addChild(topbg, TITLE_BG_LAYER)
    self:markAutoCleanupImage(titleImage)
    self.topbg = topbg
	--wm
    local title = CCLabelTTF:create(title, TITLE_FONT, TITLE_SIZE)        
    local size  = title:getContentSize()
    title:setPosition(display.cx, display.top - (size.height * 0.5))
    title:setColor(TITLE_COLOR)
    self:addChild(title, TITLE_LAYER)
    self.title = title
]]   
end
-- 修改歌曲标题，切换歌曲时用到
function CarScene:changeTitle(title)
    self.title:setString(title) 
end
-- 播放器最小化面板
function CarScene:addMiniMusicPanel(menuImage)
    local menu = display.newSprite(menuImage)
    local size = menu:getContentSize()
    local tx   = display.cx
    local ty   = size.height * 0.5
    menu:setPosition(tx, ty)
    menu:setTouchEnabled(true)
    menu:addTouchEventListener(function(event, x, y)   
        -- 点击后小菜单不可再次点击，显示播放器动作
        if event == "began" then
            menu:setVisible(false)
            menu:setTouchEnabled(false)
            
            local size = self.playBG:getContentSize()
            local tx   = display.cx
            local ty   = size.height * 0.5            
            local moveAction = CCMoveTo:create(SHOW_MUSICPANEL_TIME, CCPoint(tx,ty))

            --播放器面板展开时的操作
            transition.execute(self.playBG, moveAction, {onComplete = function()                
                -- 定义何时收回播放器面板
                self:setTouchMusicPanel(PANEL_OPEN)
                menu:scheduleUpdate(handler(self, self.onMusicPanelMonitor)) 
            end})            
            return false
        end
    end) 

    self:addChild(menu, MENU_SMALL_LAYER)
    self:markAutoCleanupImage(menuImage)
    self.menu = menu
end
-- 添加播放器
function CarScene:addMusicPlayerPanel(playBGImage, playOrderListImage, 
    playAllLoopImage, playOneLoopImage, playPrevImage, playStartImage, playPauseImage, playNextImage, playPlayListImage)

    local playBG = display.newSprite(playBGImage)
    local size   = playBG:getContentSize()
    local tx     = display.cx
    local ty     = -size.height * 0.5
    playBG.runtime = 0
    playBG:setPosition(tx, ty)

    -- 数据库中有播放歌曲时，才需要监听正在播放的歌曲状态
    if self.musiclist ~= nil then
        playBG:scheduleUpdate(handler(self, self.onMusicMonitor)) 
    end

    self:addChild(playBG, MENU_PLAY_LAYER)
    self:markAutoCleanupImage(playBGImage)
    self.playBG  = playBG

    self.playOrderList  = self:addMusicPlay(playBG, playOrderListImage, "orderlist", true, true, 82, 35)    
    self.playAllLoop    = self:addMusicPlay(playBG, playAllLoopImage,   "allloop",   true, false, 82, 35)    
    self.playOneLoop    = self:addMusicPlay(playBG, playOneLoopImage,   "oneloop",   true, false, 82, 35)
    self.playPrev       = self:addMusicPlay(playBG, playPrevImage,      "prev",      true, true, 114, 106)
    self.playPause      = self:addMusicPlay(playBG, playStartImage,     "pause",     true, false, 218, 125)
    self.playStart      = self:addMusicPlay(playBG, playPauseImage,     "start",     true, true, 218, 125)
    self.playNext       = self:addMusicPlay(playBG, playNextImage,      "next",      true, true, 307, 106)
    self.playList       = self:addMusicPlay(playBG, playPlayListImage,  "list",      true, true, 347, 35)
    self.selectedPlayer = nil
end
-- 添加播放器的播放按钮
function CarScene:addMusicPlay(parent, image, name, bTouch, bVisible, tx, ty, opacity)
    local sprite = display.newSprite(image, tx, ty)
    sprite.name = name
    sprite.touchstatus = bTouch
    if opacity ~= nil then sprite:setOpacity(opacity) end
    sprite:setVisible(bVisible)
    sprite:setTouchEnabled(bTouch)    
    sprite:addTouchEventListener(handler(self, self.onTouchEvent))
    parent:addChild(sprite)
    self:markAutoCleanupImage(image)
    return sprite
end
-- 设置播放、暂停按钮的可见状态
function CarScene:setMusicPlayStatus(status)
    if status == PLAY_STATUS then
        self.playStart:setVisible(true)
        self.playPause:setVisible(false)        
    elseif status == PAUSE_STATUS then
        self.playPause:setVisible(true)        
        self.playStart:setVisible(false)        
    end
end
-- 设置列表、列表循环、单曲循环的可见状态
function CarScene:setMusicPlayMode(mode)
    if mode == LIST_MODE then
        self.playmode = LIST_MODE
        self.playOrderList:setVisible(true)
        self.playAllLoop:setVisible(false)
        self.playOneLoop:setVisible(false)
    elseif mode == ALLLOOP_MODE then
        self.playmode = ALLLOOP_MODE
        self.playAllLoop:setVisible(true)
        self.playOrderList:setVisible(false)        
        self.playOneLoop:setVisible(false)        
    elseif mode == ONELOOP_MODE then
        self.playmode = ONELOOP_MODE
        self.playOneLoop:setVisible(true)
        self.playAllLoop:setVisible(false)
        self.playOrderList:setVisible(false)        
    end
end
-- 记忆播放器按键的状态
function CarScene:setTouchMusicPanel(mode)
    if mode == PANEL_OPEN then
        self.playOrderList:setTouchEnabled(self.playOrderList.touchstatus)
        self.playAllLoop:setTouchEnabled(self.playAllLoop.touchstatus)
        self.playOneLoop:setTouchEnabled(self.playOneLoop.touchstatus)
        self.playPrev:setTouchEnabled(self.playPrev.touchstatus)
        self.playPause:setTouchEnabled(self.playPause.touchstatus)
        self.playStart:setTouchEnabled(self.playStart.touchstatus)
        self.playList:setTouchEnabled(self.playList.touchstatus)
        self.playNext:setTouchEnabled(self.playNext.touchstatus)

    elseif mode == PANEL_CLOSE then
        self.playOrderList.touchstatus = self.playOrderList:isTouchEnabled()
        self.playOrderList:setTouchEnabled(false)

        self.playAllLoop.touchstatus = self.playAllLoop:isTouchEnabled()
        self.playAllLoop:setTouchEnabled(false)        

        self.playOneLoop.touchstatus = self.playOneLoop:isTouchEnabled()
        self.playOneLoop:setTouchEnabled(false)        

        self.playPrev.touchstatus = self.playPrev:isTouchEnabled()
        self.playPrev:setTouchEnabled(false)                

        self.playPause.touchstatus = self.playPause:isTouchEnabled()
        self.playPause:setTouchEnabled(false)                        

        self.playStart.touchstatus = self.playStart:isTouchEnabled()
        self.playStart:setTouchEnabled(false)                                

        self.playList.touchstatus = self.playList:isTouchEnabled()
        self.playList:setTouchEnabled(false)

        self.playNext.touchstatus = self.playNext:isTouchEnabled()
        self.playNext:setTouchEnabled(false)
    end
end
-- 控制播放器隐藏
function CarScene:onMusicPanelMonitor(dt)
    self.playBG.runtime = self.playBG.runtime + dt
    -- HIDE_MUSICPANEL_TIME 秒钟后没有播放器操作就隐藏播放器
    if self.playBG.runtime > HIDE_MUSICPANEL_TIME then
        local size = self.playBG:getContentSize()
        local tx   = display.cx
        local ty   = -size.height * 0.5            
        local moveAction = CCMoveTo:create(1.0, CCPoint(tx,ty))

        self:setTouchMusicPanel(PANEL_CLOSE)
        -- 播放器面板回收之后定义的操作
        transition.execute(self.playBG, moveAction, {onComplete = function()
             self.menu:setTouchEnabled(true)
             self.menu:setVisible(true)
        end})
                
        self.menu:unscheduleUpdate()
        self.playBG.runtime = 0
    end
end
-- 监控歌曲播放完成后，如何操作
function CarScene:onMusicMonitor(dt)
    if self.musiclist == nil then return end

    local bPlaying = audio.isMusicPlaying()
    if bPlaying then return end
    -- 根据播放模式，列表、列表循环、单曲循环来确定
    -- 下一首歌该怎么播
    self:doneCurrentSong(self.playmode)
end

-- 播放器按钮触摸操作
function CarScene:onTouchEvent(event, x, y)
    -- 数据库没有歌曲，所以按钮失效
    if self.musiclist == nil then return end
    local target = nil
    if event == "began" then
        target = self:getTouchTarget(x, y)
        if target == nil then 
            return false
        else 
            target:setOpacity(128)
            self.selectedPlayer = target
            return true
        end

    elseif event == "moved" then
        target = self:getTouchTarget(x, y)
        if target ~= self.selectedPlayer then
            self.selectedPlayer:setOpacity(255)
            self.selectedPlayer = nil
            return false
        else 
        end

    elseif event == "ended" and self.selectedPlayer ~= nil then
        self.selectedPlayer:setOpacity(255)
        local name = self.selectedPlayer.name
        self:onClickMusicButton(name)
        -- 按钮被操作，重新计算隐藏播放器的时间
        self.playBG.runtime = 0        
        -- 重置被选按钮
        self.selectedPlayer = nil
    end
end
-- 播放器组件确定点击操作
function CarScene:onClickMusicButton(name)
    if name == "start" then
        audio.pauseMusic()
        self:setMusicPlayStatus(PAUSE_STATUS)
    elseif name == "pause" then
        audio.resumeMusic()
        self:setMusicPlayStatus(PLAY_STATUS)
    elseif name == "prev" then
        self.currmusic = self.currmusic - 1
        if self.currmusic < self.firstMusicIdx then 
            self.currmusic = self.lastMusicIdx 
        end

        self:playSong(self.musiclist, self.currmusic)

        if self.playPause:isVisible() then
           self:setMusicPlayStatus(PLAY_STATUS)
        end
    elseif name == "next" then
        self.currmusic = self.currmusic + 1
        if self.currmusic > self.lastMusicIdx then 
            self.currmusic = self.firstMusicIdx 
        end

        self:playSong(self.musiclist, self.currmusic)

        if self.playPause:isVisible() then
           self:setMusicPlayStatus(PLAY_STATUS)
        end
    elseif name == "orderlist" then
        self:setMusicPlayMode(ALLLOOP_MODE)
    elseif name == "allloop" then
        self:setMusicPlayMode(ONELOOP_MODE)
    elseif name == "oneloop" then
        self:setMusicPlayMode(LIST_MODE)
    elseif name == "list" then
        -- TODO 跳转到APK列表页面
    end    
end
-- 获取被点击的播放器按钮
function CarScene:getTouchTarget(x,y)
    if  self.playOrderList:isTouchEnabled() and 
        self.playOrderList:isVisible() and
        self.playOrderList:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then 
        return self.playOrderList 
    elseif self.playAllLoop:isTouchEnabled() and 
        self.playAllLoop:isVisible() and
        self.playAllLoop:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playAllLoop
    elseif self.playOneLoop:isTouchEnabled() and 
        self.playOneLoop:isVisible() and
        self.playOneLoop:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playOneLoop
    elseif self.playPrev:isTouchEnabled() and 
        self.playPrev:isVisible() and
        self.playPrev:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playPrev
    elseif self.playPause:isTouchEnabled() and 
        self.playPause:isVisible() and
        self.playPause:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playPause
    elseif self.playStart:isTouchEnabled() and 
        self.playStart:isVisible() and
        self.playStart:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playStart
    elseif self.playList:isTouchEnabled() and 
        self.playList:isVisible() and
        self.playList:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playList
    elseif self.playNext:isTouchEnabled() and 
        self.playNext:isVisible() and
        self.playNext:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playNext                        
    else
        return nil
    end
end

function CarScene:destruct()
    if self.musicinit then
        self:removeMusicPlayer()
    end

    if self.gameinit then
        self:removeGameContent()
    end

    if self.bginit then
        transition.stopTarget(self.background)
        self.background:removeSelf(true)
    end

    audio.stopAllSounds()
    if audio.isMusicPlaying() then audio.stopMusic(true) end

    transition.stopTarget(self)
    self:removeAllChildrenWithCleanup(true)
end

function CarScene:removeGameContent()
    for nodeid = 1, #NODES, 1 do
        local node = NODES[nodeid]
        local movecars = node.movecars        
        for moveid = 1, #movecars, 1 do
            local movecar = movecars[moveid]
            transition.stopTarget(movecar)
            self:removeTouchTarget(movecar, true)
            movecars[moveid] = 0            
        end
        node.movecars = nil

        local stopcars = node.stopcars        
        for stopid = 1, #stopcars, 1 do
            local stopcar = stopcars[stopid]           
            transition.stopTarget(stopcar)
            self:removeTouchTarget(stopcar, true)
            stopcars[stopid] = 0
        end

        node.stopcars = nil
        node:removeSelf(true)
        NODES[nodeid] = 0
    end

    NODES = nil 
end
-- 场景中删除播放器
function CarScene:removeMusicPlayer()
    transition.stopTarget(self.menu)
    self.menu:unscheduleUpdate()
    self:removeTouchTarget(self.menu, true)
    self.menu = nil

    self:removeTouchTarget(self.playOrderList, true)
    self.playOrderList = nil

    self:removeTouchTarget(self.playAllLoop, true)
    self.playAllLoop = nil

    self:removeTouchTarget(self.playOneLoop, true)
    self.playOneLoop = nil    

    self:removeTouchTarget(self.playPrev, true)
    self.playPrev = nil     

    self:removeTouchTarget(self.playPause, true)
    self.playPause = nil

    self:removeTouchTarget(self.playStart, true)
    self.playStart = nil

    self:removeTouchTarget(self.playNext, true)
    self.playNext = nil     

    self:removeTouchTarget(self.playList, true)
    self.playList = nil     
    self.selectedPlayer = nil

    transition.stopTarget(self.playBG)
    self.playBG:unscheduleUpdate()
    self:removeTouchTarget(self.playBG, true)
    self.playBG = nil    


    -- 虽然self.title继承CCSprite，
    -- 但是使用self.title:removeSelf(true)会报错，原因何在？
    self.title:removeFromParentAndCleanup(true)
    self.title = nil
              
    -- self.topbg:removeSelf(true)
    -- self.topbg = nil     
    self.musiclist = nil      
end
-- 删除播放按钮
function CarScene:removeTouchTarget(sprite, bclean)
    sprite:setTouchEnabled(false)
    sprite:removeTouchEventListener()
    sprite:removeSelf(bclean)
end
--没有可播放歌曲提示窗口
--[[
function CarScene:addDialogBox(bgImage, fgImage, btnImage)
    local node = display.newNode()
    self:addChild(node, DIALOG_LAYER)
    self.dialog = node

    local bg = display.newSprite(bgImage, display.cx, display.cy)
    node:addChild(bg)
    node.bg = bg

    local fg = display.newSprite(fgImage, display.cx, display.cy)
    node:addChild(fg)
    node.fg = fg

    local btn = display.newSprite(btnImage, display.cx + 100, display.cy)
    node:addChild(btn)
    node.btn = btn

    local msg = CCLabelTTF:create(DIALOG_MSG, TITLE_FONT, TITLE_SIZE)
    msg:setPosition(display.cx, display.cy - 100)
    msg:setColor(DIALOG_COLOR)
    node:addChild(msg)
    node.msg = msg

    node:setTouchEnabled(true)
    node:addTouchEventListener(function(event, x, y) 
        if event == "began" then
            local touchIn = btn:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y))
            if not touchIn then return cc.TOUCH_BEGAN_SWALLOWS end

            self:removeDialogBox(bgImage, fgImage, btnImage)
            return cc.TOUCH_BEGAN_SWALLOWS
        elseif event == "moved" then
        elseif event == "ended" then            
        else            
        end 
    end)
end
]]--
function CarScene:removeDialogBox(bgImage, fgImage, btnImage)
    self.dialog.msg:removeFromParentAndCleanup(true)
    self.dialog.msg = nil
    self.dialog.btn:removeSelf(true)
    self.dialog.btn = nil
    self.dialog.fg:removeSelf(true)
    self.dialog.fg = nil
    self.dialog.bg:removeSelf(true)
    self.dialog.bg = nil
    self.dialog:setTouchEnabled(false)
    self.dialog:removeTouchEventListener()    
    self.dialog:removeSelf(true)
    self.dialog = nil

    self:markAutoCleanupImage(bgImage)
    self:markAutoCleanupImage(fgImage)
    self:markAutoCleanupImage(btnImage)
end
return CarScene