local random = math.random

local NODES = nil

local cloudCount = 10
local planeCount = 2
-- 随机位置偏移值
local offset     = 50
-- 飞机的宽高的一半
local halfPlane  = 125
-- 飞行员的宽高的一半
local halfFlyer  = 100
local distFlyCloud = 1500

local cartoonNames  = {"red_flyer", "green_flyer", "red_plane", "green_plane"}
local flyers        = {"red_flyer", "green_flyer"}
local planes        = {"red_plane", "green_plane"}
local cartoonFrames = {6,6,6,6}
local cartoonTimes  = {1/30, 1/30, 1/60, 1/60}
local cloudTimes    = {4.5, 5.0, 5.5, 6.0, 6.5, 7.0, 8.0, 8.5, 9.0, 9.5, 10, 12, 14}
local flightTimes   = {2.5, 3.0, 3.5, 4.0, 4.5, 5.0, 5.5}
local flightDelays  = {0.3, 0.5, 0.8, 1.2, 1.5}


-- 飞行员默认屏幕外的位置
local flyerDefault = -300
-- 飞行员上升的高度
local flyerHeight = 300
-- 飞行员飞行状态
local FLYER_STATUS = "flyer"
local EMPTY_STATUS = "empty"

local PlaneScene = class("PlaneScene", function()
    return display.newScene("PlaneScene")
end)

function PlaneScene:ctor()
    self:addBG(BG_PLANE)
end

function PlaneScene:onEnter()
    if device.platform == "android" then
        -- avoid unmeant back
        self:performWithDelay(function()
            -- keypad layer, for android
            local layer = display.newLayer()
            layer:addKeypadEventListener(function(event)
                if event == "back" then app.exit() end
            end)
            layer:setKeypadEnabled(true)
            self:addChild(layer)
        end, 0.5)
    end
end

function PlaneScene:onInit()
    display.addSpriteFramesWithFile(PLANE[1][1], PLANE[1][2])
    self:markAutoCleanupImage(PLANE[1][2])
    display.addSpriteFramesWithFile(PLANE[2][1], PLANE[2][2])
    self:markAutoCleanupImage(PLANE[2][2])
    display.addSpriteFramesWithFile(PLANE[3][1], PLANE[3][2])
    self:markAutoCleanupImage(PLANE[3][2])
    display.addSpriteFramesWithFile(PLANE[4][1], PLANE[4][2])
    self:markAutoCleanupImage(PLANE[4][2])
    -- self:markAutoCleanupImage(PLANE_SMOKE)

    local house = display.newSprite("house.png")
    local size  = house:getContentSize()
    house:setPosition(display.cx, size.height * 0.5)
    self:addChild(house)
    self:markAutoCleanupImage("house.png")    
    self.house = house

    -- self:addSmokeNode()
    self:addNodeAndCache()
    self:initScenePlanes(planeCount)
    self.gameinit = true

    -- 音乐播放器初始化
    self.musiclist = self:addMusicList(DB_PATH, SQL_QUERY)
    if self.musiclist == nil then 
        self:addDialogBox(DIALOG_BG, DIALOG_FG, DIALOG_BTN)
    else
        self.firstMusicIdx = 1
        self.lastMusicIdx = #self.musiclist
        self.currmusic = 1
        self.playmode = LIST_MODE
        self:playSong(self.musiclist, self.currmusic)
    end
        
    --self:addMiniMusicPanel(MENU_SMALL)
    --self:addMusicPlayerPanel(MENU_PLAY, MENU_ORDER_LIST, MENU_ALL_LOOP, MENU_ONE_LOOP, MENU_PREV, MENU_START, MENU_PAUSE, MENU_NEXT, MENU_PLAYLIST)
    self.musicinit = true   
end

function PlaneScene:onExit()

end
-- 场景清理资源时自动调用此方法
function PlaneScene:onCleanup()
    self:destruct()
end
-- 添加场景背景
function PlaneScene:addBG(bgImage)
    -- display.removeSpriteFrameByImageName(bgImage)
    display.addImageAsync(bgImage, function(event, texture)
        local bg = display.newSprite(bgImage, display.cx, display.cy)
        bg:setOpacity(0)
        self:addChild(bg)
        self:markAutoCleanupImage(bgImage)
        self.background = bg
        self.bginit = true

        local action = CCFadeIn:create(FADEIN_TIME)
        transition.execute(bg, action, {onComplete = function() self:onInit() end})
    end)
end
-- 新建并添加渲染节点和缓存
function PlaneScene:addNodeAndCache()
    NODES = {}
    for idx = 1, #cartoonNames, 1 do 
        self:addRenderNode(idx)
        self:setCartoonCache(idx)
    end    
end
-- 渲染节点
function PlaneScene:addRenderNode(nodeID)
    local image = PLANE[nodeID][2]
    local node  = display.newBatchNode(image)
    self:addChild(node)
    table.insert(NODES, node)
end
-- 设置动画缓存
function PlaneScene:setCartoonCache(cacheID)
    local pattern     = cartoonNames[cacheID] .. "_%02d.png"    
    local frameLength = cartoonFrames[cacheID]
    local frames      = display.newFrames(pattern, 1, frameLength)
    local time        = cartoonTimes[cacheID]
    local animation   = display.newAnimation(frames, time)
    local cacheName   = cartoonNames[cacheID]
    display.setAnimationCache(cacheName, animation)    
end

function PlaneScene:addSmokeNode()
    local node = display.newNode()
    self:addChild(node)
    self.smoke = node

    self.smoke:scheduleUpdate(handler(self, self.onSmokeEvent)) 
end

function PlaneScene:onSmokeEvent(dt)
    local children = self.smoke:getChildren()
    if children == nil then return end

    for i = 0, children:count() - 1, 1 do
        local child = tolua.cast(children:objectAtIndex(i), "CCParticleSmoke")
        -- CCParticleSystemQuad
        -- CCParticleSystem
        local size  = child.plane:getContentSize()
        local tx,ty = child.plane:getPosition()

        if child.plane:isFlipX() then 
            tx = tx + size.width * 0.5
            ty = ty + size.height * 0.5
        else
            tx = tx - size.width * 0.5
            ty = ty + size.height * 0.5
        end

        child:setPosition(tx,ty)
    end    
end
-- 初始化场景中的飞机
function PlaneScene:initScenePlanes(planeCount)
    for idx = 1, planeCount, 1 do
        local nodeid = random(1, #planes)
        local node   = NODES[nodeid]
        local plane  = self:newNodePlane(nodeid)
    end
end

-- 新建飞行员
function PlaneScene:newNodeFlyer(nodeID, x, y)
    local cacheName = flyers[nodeID]
    local flyerName = "#" .. cacheName .. "_01.png"
    local flyer  = display.newSprite(flyerName, x, y)
    flyer.state  = STOP_STATUS
    local parent = NODES[nodeID]
    parent:addChild(flyer)

    return flyer
end
-- 新建飞机
function PlaneScene:newNodePlane(nodeID)
    local cacheName = planes[nodeID]
    local planeName = "#" .. cacheName .. "_01.png"
    local lrIndex   = random(0, 1)
    local tbIndex   = 0
    local tx, ty    = self:getPlanePos(lrIndex, tbIndex)    
    local plane = display.newSprite(planeName, tx, ty)
    plane.state = FLYER_STATUS
    plane:setTouchEnabled(true)
    plane:playForever(display.getAnimationCache(cacheName))
    plane:addTouchEventListener(function(event, x, y)
        if event == "began" then
            local touchIn = plane:getCascadeBoundingBox():containsPoint(CCPoint(x, y))
            if plane.state == FLYER_STATUS and touchIn then 
                plane.state  = EMPTY_STATUS
                local tx, ty = plane:getPosition()
                local flyer  = self:newNodeFlyer(nodeID)
                self:skydiving(nodeID, flyer, tx, ty)
            end
            return cc.TOUCH_BEGAN_SWALLOWS    
        end
    end)

    local parent = NODES[nodeID + 2]
    parent:addChild(plane)
    self:flightingTo(plane)

    return plane
end
-- 获取飞机的位置
function PlaneScene:getPlanePos(lrIndex, tbIndex)
    if lrIndex == 0 and tbIndex == 0 then
        return self:getTopLeftPos()
    elseif lrIndex == 0 and tbIndex == 1 then
        return self:getBottomLeftPos()
    elseif lrIndex == 1 and tbIndex == 0 then
        return self:getTopRightPos()
    elseif lrIndex == 1 and tbIndex == 1 then
        return self:getBottomRightPos()
    end
end
-- 获取左上角位置
function PlaneScene:getTopLeftPos()
    local tx = display.left - halfPlane
    local ty = display.top + random(-offset, offset)
-- TEST:
    -- local tx = display.left + halfPlane
    -- local ty = display.top - offset
    return tx, ty
end
-- 获取右上角位置
function PlaneScene:getTopRightPos()
    local tx = display.right + halfPlane
    local ty = display.top + random(-offset, offset)
-- TEST:
    -- local tx = display.right - halfPlane
    -- local ty = display.top - offset
    return tx, ty    
end
-- 获取左下角位置
function PlaneScene:getBottomLeftPos()
    local tx = display.left - halfPlane - 200
    local ty = display.bottom + random(-offset, offset) - 200
-- TEST:
    -- local tx = display.left + halfPlane
    -- local ty = display.bottom + offset
    return tx, ty    
end
-- 获取右上角位置
function PlaneScene:getBottomRightPos()
    local tx = display.right + halfPlane + 200
    local ty = display.bottom + random(-offset, offset) - 200
-- TEST:
    -- local tx = display.right - halfPlane
    -- local ty = display.bottom + offset
    return tx, ty
end

-- 飞行动作
function PlaneScene:flightingTo(plane)
    local delayTime = flightDelays[random(1, #flightDelays)]
    self:performWithDelay(function()     
        local tx, ty = plane:getPosition()                
        local lrIndex = 0
        local tbIndex = 0

        if tx > display.cx then             
            if ty < display.cy then 
                tbIndex = 0
                plane:setRotation(90)                 
            else 
                tbIndex = 1
                plane:setRotation(0) 
            end

            lrIndex = 0
            plane:flipX(true) 
        else
            if ty < display.cy then 
                tbIndex = 0
                plane:setRotation(-90) 
            else 
                tbIndex = 1
                plane:setRotation(0)
            end

            lrIndex = 1
            plane:flipX(false) 
        end
        --[[ //粒子系统添加
        local smoke = CCParticleSmoke:create()
        smoke:setLife(0.8)
        local red = random(0.1, 1.0)
        local green = random(0.1, 1.0)
        local blue = random(0.1, 1.0)
        smoke:setStartColor(ccc4f(red, green, blue, 0.5))
        smoke:setEndColor(ccc4f(red, green, blue, 0.15))
        smoke.plane = plane
        smoke:setTexture(CCTextureCache:sharedTextureCache():addImage(PLANE_SMOKE))
        self.smoke:addChild(smoke)        
        plane.smoke = smoke
        ]]
        local actionTime = flightTimes[random(1, #flightTimes)]
        tx, ty = self:getPlanePos(lrIndex, tbIndex)
        local action = CCMoveTo:create(actionTime, CCPoint(tx, ty))
        
        -- 飞机飞出屏幕完成后的操作
        transition.execute(plane, action, {onComplete = function()
            local nodeid = random(1, #planes)
            local node = NODES[nodeid]
            local newPlane = self:newNodePlane(nodeid)

            self:performWithDelay(function()
                --[[ 粒子系统清除
                plane.smoke.plane = nil
                plane.smoke:removeFromParentAndCleanup(true)
                plane.smoke = nil
                ]]
                transition.stopTarget(plane)
                self:removeTouchTarget(plane, true)
            end, 3.0)
        end})     
    end, delayTime)    
end
-- 跳伞动作
function PlaneScene:skydiving(flyerID, flyer, tx, ty)
    local cacheName = flyers[flyerID]    
    flyer:playForever(display.getAnimationCache(cacheName))
    flyer:setPosition(tx,ty)

    local dist = ty + flyerHeight
    local speed = 150
    local time = flyerHeight / speed
    local actionOne = CCMoveTo:create(time, CCPoint(tx, dist))

    dist = dist - (-halfFlyer)
    speed = 50
    time = dist / speed
    local actionTwo = CCMoveTo:create(time, CCPoint(tx, -halfFlyer))
    local actions = transition.sequence({actionOne, actionTwo})

    -- 飞行员跳伞完成后的操作
    transition.execute(flyer, actions, {onComplete = function()     
        transition.stopTarget(flyer)
        flyer:removeSelf(true)               
    end})    
end

-- 从数据库中读取播放列表
function PlaneScene:addMusicList(dburl, sqlquery)
    if not io.exists(dburl) then return nil end
    local sqlite3 = require("lsqlite3")
    local db = sqlite3.open(dburl)
    local playlist = {}
    for folder, file, song in db:urows(sqlquery) do
        table.insert(playlist, {folder, file, song})
    end        
    db:close()
    -- 数据库可能出现没有歌曲的情况
    if #playlist == 0 then playlist = nil end
    return playlist
end
-- 播放歌曲
function PlaneScene:playSong(list, index)
    if list == nil then return end

    local songName = list[index][3]
    if self.title == nil then 
        self:addTitle(TOP_BG, songName)
    else
        self:changeTitle(songName)
    end

    local folderPath = list[index][1]
    -- audio.preloadMusic(folderPath)
    local songFile = folderPath .. list[index][2]
	--wm
    --audio.preloadMusic(songFile)
    --audio.playMusic(songFile, false)
end
-- 播放完成当前歌曲后，该如何操作
function PlaneScene:doneCurrentSong(mode)
    if mode == LIST_MODE then
        self.currmusic = self.currmusic + 1
        if self.currmusic > self.lastMusicIdx then
            self.currmusic = self.lastMusicIdx
            audio.stopMusic()
        else
            self:playSong(self.musiclist, self.currmusic)
        end

    elseif mode == ALLLOOP_MODE then
        self.currmusic = self.currmusic + 1
        if self.currmusic > self.lastMusicIdx then 
            self.currmusic = self.firstMusicIdx 
        end
        self:playSong(self.musiclist, self.currmusic)
    elseif mode == ONELOOP_MODE then
        self:playSong(self.musiclist, self.currmusic)
    end    
end
-- 添加歌曲标题
function PlaneScene:addTitle(titleImage, title)
--[[
    local topbg = display.newSprite(titleImage)
    local size  = topbg:getContentSize()
    topbg:setPosition(display.left, display.top - (size.height * 0.5))
    topbg:setScaleX((display.width / size.width) * 2)    
    self:addChild(topbg, TITLE_BG_LAYER)
    self:markAutoCleanupImage(titleImage)
    self.topbg = topbg
	--wm
    local title = CCLabelTTF:create(title, TITLE_FONT, TITLE_SIZE)        
    local size  = title:getContentSize()
    title:setPosition(display.cx, display.top - (size.height * 0.5))
    title:setColor(TITLE_COLOR)
    self:addChild(title, TITLE_LAYER)
    self.title = title
]]    
end
-- 修改歌曲标题，切换歌曲时用到
function PlaneScene:changeTitle(title)
    self.title:setString(title) 
end
-- 播放器最小化面板
function PlaneScene:addMiniMusicPanel(menuImage)
    local menu = display.newSprite(menuImage)
    local size = menu:getContentSize()
    local tx   = display.cx
    local ty   = size.height * 0.5
    menu:setPosition(tx, ty)
    menu:setTouchEnabled(true)
    menu:addTouchEventListener(function(event, x, y)   
        -- 点击后小菜单不可再次点击，显示播放器动作
        if event == "began" then
            menu:setVisible(false)
            menu:setTouchEnabled(false)
            
            local size = self.playBG:getContentSize()
            local tx   = display.cx
            local ty   = size.height * 0.5            
            local moveAction = CCMoveTo:create(SHOW_MUSICPANEL_TIME, CCPoint(tx,ty))

            --播放器面板展开时的操作
            transition.execute(self.playBG, moveAction, {onComplete = function()                
                -- 定义何时收回播放器面板
                self:setTouchMusicPanel(PANEL_OPEN)
                menu:scheduleUpdate(handler(self, self.onMusicPanelMonitor)) 
            end})            
            return false
        end
    end) 

    self:addChild(menu, MENU_SMALL_LAYER)
    self:markAutoCleanupImage(menuImage)
    self.menu = menu
end
-- 添加播放器
function PlaneScene:addMusicPlayerPanel(playBGImage, playOrderListImage, 
    playAllLoopImage, playOneLoopImage, playPrevImage, playStartImage, playPauseImage, playNextImage, playPlayListImage)

    local playBG = display.newSprite(playBGImage)
    local size   = playBG:getContentSize()
    local tx     = display.cx
    local ty     = -size.height * 0.5
    playBG.runtime = 0
    playBG:setPosition(tx, ty)

    -- 数据库中有播放歌曲时，才需要监听正在播放的歌曲状态
    if self.musiclist ~= nil then
        playBG:scheduleUpdate(handler(self, self.onMusicMonitor)) 
    end

    self:addChild(playBG, MENU_PLAY_LAYER)
    self:markAutoCleanupImage(playBGImage)
    self.playBG  = playBG

    self.playOrderList  = self:addMusicPlay(playBG, playOrderListImage, "orderlist", true, true, 82, 35)    
    self.playAllLoop    = self:addMusicPlay(playBG, playAllLoopImage,   "allloop",   true, false, 82, 35)    
    self.playOneLoop    = self:addMusicPlay(playBG, playOneLoopImage,   "oneloop",   true, false, 82, 35)
    self.playPrev       = self:addMusicPlay(playBG, playPrevImage,      "prev",      true, true, 114, 106)
    self.playPause      = self:addMusicPlay(playBG, playStartImage,     "pause",     true, false, 218, 125)
    self.playStart      = self:addMusicPlay(playBG, playPauseImage,     "start",     true, true, 218, 125)
    self.playNext       = self:addMusicPlay(playBG, playNextImage,      "next",      true, true, 307, 106)
    self.playList       = self:addMusicPlay(playBG, playPlayListImage,  "list",      true, true, 347, 35)
    self.selectedPlayer = nil
end
-- 添加播放器的播放按钮
function PlaneScene:addMusicPlay(parent, image, name, bTouch, bVisible, tx, ty, opacity)
    local sprite = display.newSprite(image, tx, ty)
    sprite.name = name
    sprite.touchstatus = bTouch
    if opacity ~= nil then sprite:setOpacity(opacity) end
    sprite:setVisible(bVisible)
    sprite:setTouchEnabled(bTouch)    
    sprite:addTouchEventListener(handler(self, self.onTouchEvent))
    parent:addChild(sprite)
    self:markAutoCleanupImage(image)
    return sprite
end
-- 设置播放、暂停按钮的可见状态
function PlaneScene:setMusicPlayStatus(status)
    if status == PLAY_STATUS then
        self.playStart:setVisible(true)
        self.playPause:setVisible(false)        
    elseif status == PAUSE_STATUS then
        self.playPause:setVisible(true)        
        self.playStart:setVisible(false)        
    end
end
-- 设置列表、列表循环、单曲循环的可见状态
function PlaneScene:setMusicPlayMode(mode)
    if mode == LIST_MODE then
        self.playmode = LIST_MODE
        self.playOrderList:setVisible(true)
        self.playAllLoop:setVisible(false)
        self.playOneLoop:setVisible(false)
    elseif mode == ALLLOOP_MODE then
        self.playmode = ALLLOOP_MODE
        self.playAllLoop:setVisible(true)
        self.playOrderList:setVisible(false)        
        self.playOneLoop:setVisible(false)        
    elseif mode == ONELOOP_MODE then
        self.playmode = ONELOOP_MODE
        self.playOneLoop:setVisible(true)
        self.playAllLoop:setVisible(false)
        self.playOrderList:setVisible(false)        
    end
end
-- 记忆播放器按键的状态
function PlaneScene:setTouchMusicPanel(mode)
    if mode == PANEL_OPEN then
        self.playOrderList:setTouchEnabled(self.playOrderList.touchstatus)
        self.playAllLoop:setTouchEnabled(self.playAllLoop.touchstatus)
        self.playOneLoop:setTouchEnabled(self.playOneLoop.touchstatus)
        self.playPrev:setTouchEnabled(self.playPrev.touchstatus)
        self.playPause:setTouchEnabled(self.playPause.touchstatus)
        self.playStart:setTouchEnabled(self.playStart.touchstatus)
        self.playList:setTouchEnabled(self.playList.touchstatus)
        self.playNext:setTouchEnabled(self.playNext.touchstatus)

    elseif mode == PANEL_CLOSE then
        self.playOrderList.touchstatus = self.playOrderList:isTouchEnabled()
        self.playOrderList:setTouchEnabled(false)

        self.playAllLoop.touchstatus = self.playAllLoop:isTouchEnabled()
        self.playAllLoop:setTouchEnabled(false)        

        self.playOneLoop.touchstatus = self.playOneLoop:isTouchEnabled()
        self.playOneLoop:setTouchEnabled(false)        

        self.playPrev.touchstatus = self.playPrev:isTouchEnabled()
        self.playPrev:setTouchEnabled(false)                

        self.playPause.touchstatus = self.playPause:isTouchEnabled()
        self.playPause:setTouchEnabled(false)                        

        self.playStart.touchstatus = self.playStart:isTouchEnabled()
        self.playStart:setTouchEnabled(false)                                

        self.playList.touchstatus = self.playList:isTouchEnabled()
        self.playList:setTouchEnabled(false)

        self.playNext.touchstatus = self.playNext:isTouchEnabled()
        self.playNext:setTouchEnabled(false)
    end
end
-- 控制播放器隐藏
function PlaneScene:onMusicPanelMonitor(dt)
    self.playBG.runtime = self.playBG.runtime + dt
    -- HIDE_MUSICPANEL_TIME 秒钟后没有播放器操作就隐藏播放器
    if self.playBG.runtime > HIDE_MUSICPANEL_TIME then
        local size = self.playBG:getContentSize()
        local tx   = display.cx
        local ty   = -size.height * 0.5            
        local moveAction = CCMoveTo:create(1.0, CCPoint(tx,ty))

        self:setTouchMusicPanel(PANEL_CLOSE)
        -- 播放器面板回收之后定义的操作
        transition.execute(self.playBG, moveAction, {onComplete = function()
             self.menu:setTouchEnabled(true)
             self.menu:setVisible(true)
        end})
                
        self.menu:unscheduleUpdate()
        self.playBG.runtime = 0
    end
end
-- 监控歌曲播放完成后，如何操作
function PlaneScene:onMusicMonitor(dt)
    if self.musiclist == nil then return end

    local bPlaying = audio.isMusicPlaying()
    if bPlaying then return end
    -- 根据播放模式，列表、列表循环、单曲循环来确定
    -- 下一首歌该怎么播
    self:doneCurrentSong(self.playmode)
end

-- 播放器按钮触摸操作
function PlaneScene:onTouchEvent(event, x, y)
    -- 数据库没有歌曲，所以按钮失效
    if self.musiclist == nil then return end
    local target = nil
    if event == "began" then
        target = self:getTouchTarget(x, y)
        if target == nil then 
            return false
        else 
            target:setOpacity(128)
            self.selectedPlayer = target
            return true
        end

    elseif event == "moved" then
        target = self:getTouchTarget(x, y)
        if target ~= self.selectedPlayer then
            self.selectedPlayer:setOpacity(255)
            self.selectedPlayer = nil
            return false
        else 
        end

    elseif event == "ended" and self.selectedPlayer ~= nil then
        self.selectedPlayer:setOpacity(255)
        local name = self.selectedPlayer.name
        self:onClickMusicButton(name)
        -- 按钮被操作，重新计算隐藏播放器的时间
        self.playBG.runtime = 0        
        -- 重置被选按钮
        self.selectedPlayer = nil
    end
end
-- 播放器组件确定点击操作
function PlaneScene:onClickMusicButton(name)
    if name == "start" then
        audio.pauseMusic()
        self:setMusicPlayStatus(PAUSE_STATUS)
    elseif name == "pause" then
        audio.resumeMusic()
        self:setMusicPlayStatus(PLAY_STATUS)
    elseif name == "prev" then
        self.currmusic = self.currmusic - 1
        if self.currmusic < self.firstMusicIdx then 
            self.currmusic = self.lastMusicIdx 
        end

        self:playSong(self.musiclist, self.currmusic)

        if self.playPause:isVisible() then
           self:setMusicPlayStatus(PLAY_STATUS)
        end
    elseif name == "next" then
        self.currmusic = self.currmusic + 1
        if self.currmusic > self.lastMusicIdx then 
            self.currmusic = self.firstMusicIdx 
        end

        self:playSong(self.musiclist, self.currmusic)

        if self.playPause:isVisible() then
           self:setMusicPlayStatus(PLAY_STATUS)
        end
    elseif name == "orderlist" then
        self:setMusicPlayMode(ALLLOOP_MODE)
    elseif name == "allloop" then
        self:setMusicPlayMode(ONELOOP_MODE)
    elseif name == "oneloop" then
        self:setMusicPlayMode(LIST_MODE)
    elseif name == "list" then
        -- TODO 跳转到APK列表页面
    end    
end
-- 获取被点击的播放器按钮
function PlaneScene:getTouchTarget(x,y)
    if  self.playOrderList:isTouchEnabled() and 
        self.playOrderList:isVisible() and
        self.playOrderList:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then 
        return self.playOrderList 
    elseif self.playAllLoop:isTouchEnabled() and 
        self.playAllLoop:isVisible() and
        self.playAllLoop:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playAllLoop
    elseif self.playOneLoop:isTouchEnabled() and 
        self.playOneLoop:isVisible() and
        self.playOneLoop:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playOneLoop
    elseif self.playPrev:isTouchEnabled() and 
        self.playPrev:isVisible() and
        self.playPrev:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playPrev
    elseif self.playPause:isTouchEnabled() and 
        self.playPause:isVisible() and
        self.playPause:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playPause
    elseif self.playStart:isTouchEnabled() and 
        self.playStart:isVisible() and
        self.playStart:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playStart
    elseif self.playList:isTouchEnabled() and 
        self.playList:isVisible() and
        self.playList:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playList
    elseif self.playNext:isTouchEnabled() and 
        self.playNext:isVisible() and
        self.playNext:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y)) then
        return self.playNext                        
    else
        return nil
    end
end

function PlaneScene:destruct()
    if self.musicinit then
        self:removeMusicPlayer()
    end

    if self.gameinit then
        self:removeGameContent()
    end

    if self.bginit then
        transition.stopTarget(self.background)
        self.background:removeSelf(true)
    end

    audio.stopAllSounds()
    if audio.isMusicPlaying() then audio.stopMusic(true) end

    --[[ 清除粒子层
    self.smoke:unscheduleUpdate()
    self.smoke:removeSelf(true)
    self.smoke = nil
    ]]
    transition.stopTarget(self)
    self:removeAllChildrenWithCleanup(true)
end

function PlaneScene:removeGameContent()
    for nodeid = 1, #NODES, 1 do
        local node = NODES[nodeid]
        local children = node:getChildren()
        for i = children:count() - 1, 0, -1 do
            local child = tolua.cast(children:objectAtIndex(i), "CCNode")   
            --[[清除还在运行的粒子
            if child.smoke then 
                child.smoke.plane = nil
                child.smoke:removeFromParentAndCleanup(true)
                child.smoke = nil
            end
            ]]         
            transition.stopTarget(child)
            self:removeTouchTarget(child, true)     
        end
        children:removeAllObjects()
        NODES[nodeid] = 0
    end

    NODES = nil
end
-- 场景中删除播放器
function PlaneScene:removeMusicPlayer()
    transition.stopTarget(self.menu)
    self.menu:unscheduleUpdate()
    self:removeTouchTarget(self.menu, true)
    self.menu = nil

    self:removeTouchTarget(self.playOrderList, true)
    self.playOrderList = nil

    self:removeTouchTarget(self.playAllLoop, true)
    self.playAllLoop = nil

    self:removeTouchTarget(self.playOneLoop, true)
    self.playOneLoop = nil    

    self:removeTouchTarget(self.playPrev, true)
    self.playPrev = nil     

    self:removeTouchTarget(self.playPause, true)
    self.playPause = nil

    self:removeTouchTarget(self.playStart, true)
    self.playStart = nil

    self:removeTouchTarget(self.playNext, true)
    self.playNext = nil     

    self:removeTouchTarget(self.playList, true)
    self.playList = nil     
    self.selectedPlayer = nil

    transition.stopTarget(self.playBG)
    self.playBG:unscheduleUpdate()
    self:removeTouchTarget(self.playBG, true)
    self.playBG = nil    


    -- 虽然self.title继承CCSprite，
    -- 但是使用self.title:removeSelf(true)会报错，原因何在？
    self.title:removeFromParentAndCleanup(true)
    self.title = nil
              
    -- self.topbg:removeSelf(true)
    -- self.topbg = nil    
    self.musiclist = nil       
end
-- 删除播放按钮
function PlaneScene:removeTouchTarget(sprite, bclean)
    sprite:setTouchEnabled(false)
    sprite:removeTouchEventListener()
    sprite:removeSelf(bclean)
end
--没有可播放歌曲提示窗口
--[[
function PlaneScene:addDialogBox(bgImage, fgImage, btnImage)
    local node = display.newNode()
    self:addChild(node, DIALOG_LAYER)
    self.dialog = node

    local bg = display.newSprite(bgImage, display.cx, display.cy)
    node:addChild(bg)
    node.bg = bg

    local fg = display.newSprite(fgImage, display.cx, display.cy)
    node:addChild(fg)
    node.fg = fg

    local btn = display.newSprite(btnImage, display.cx + 100, display.cy)
    node:addChild(btn)
    node.btn = btn

    local msg = CCLabelTTF:create(DIALOG_MSG, TITLE_FONT, TITLE_SIZE)
    msg:setPosition(display.cx, display.cy - 100)
    msg:setColor(DIALOG_COLOR)
    node:addChild(msg)
    node.msg = msg

    node:setTouchEnabled(true)
    node:addTouchEventListener(function(event, x, y) 
        if event == "began" then
            local touchIn = btn:getCascadeBoundingBox(true):containsPoint(CCPoint(x, y))
            if not touchIn then return cc.TOUCH_BEGAN_SWALLOWS end

            self:removeDialogBox(bgImage, fgImage, btnImage)
            return cc.TOUCH_BEGAN_SWALLOWS
        elseif event == "moved" then
        elseif event == "ended" then            
        else            
        end 
    end)
end
]]--
function PlaneScene:removeDialogBox(bgImage, fgImage, btnImage)
    self.dialog.msg:removeFromParentAndCleanup(true)
    self.dialog.msg = nil
    self.dialog.btn:removeSelf(true)
    self.dialog.btn = nil
    self.dialog.fg:removeSelf(true)
    self.dialog.fg = nil
    self.dialog.bg:removeSelf(true)
    self.dialog.bg = nil
    self.dialog:setTouchEnabled(false)
    self.dialog:removeTouchEventListener()    
    self.dialog:removeSelf(true)
    self.dialog = nil

    self:markAutoCleanupImage(bgImage)
    self:markAutoCleanupImage(fgImage)
    self:markAutoCleanupImage(btnImage)
end
return PlaneScene
